<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Tabulatable\RowFormatter;

use apexl\Io\modules\tabulator\Dto\RowFormatterDto\BooleanTabulatorRowFormatterDto;

/**
 * $dto->withRowFormatter(BooleanRowFormatter::class, [
 *     new BooleanRowFormatterDto(test: true, styles: ['backgroundColor' => '#e6ffd1']),
 *     new BooleanRowFormatterDto(test: false, styles: ['backgroundColor' => '#fac8ca']),
 * ])
 */
final readonly class BooleanRowFormatter extends RowFormatter
{

    protected function dtoClass(): string
    {
        return BooleanTabulatorRowFormatterDto::class;
    }
}