<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Dto\RowFormatterDto;

final readonly class BooleanTabulatorRowFormatterDto extends TabulatorRowFormatterDto
{
    public function __construct(
        public bool $test,
        array $rowStyles = [],
        array $rowClasses = [],
        array $cellStyles = [],
        array $cellClasses = [],
    ) {
        parent::__construct($rowStyles, $rowClasses, $cellStyles, $cellClasses);
    }
}