<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Tabulatable\RowFormatter;

use apexl\Io\modules\tabulator\Dto\RowFormatterDto\BetweenRowFormatterDto;

/**
 * $dto->withRowFormatter(BetweenRowFormatter::class, [
 *     new BetweenRowFormatterDto(from: 2023, styles: ['backgroundColor' => '#f0f']),
 *     new BetweenRowFormatterDto(from: 2021, to: 2022, styles: ['backgroundColor' => '#00f']),
 *     new BetweenRowFormatterDto(from: null, to: 2019, styles: ['backgroundColor' => '#f00']),
 * ])
 */
readonly class BetweenRowFormatter extends RowFormatter
{
    protected function dtoClass(): string
    {
        return BetweenRowFormatterDto::class;
    }
}