<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Components;

abstract readonly class AggregateTabulator extends Tabulator
{
    protected const bool DISABLE_HEADER_SORT = true;

    public function init(): void
    {
        parent::init();

        $this->setPagination(false);
        $this->setPersistence(false);
    }

    public function viewId(): string
    {
        return $this::class;
    }

    protected function addFooterRows(int $count = 1): void
    {
        $this->setProperty('footerRowCount', $count);
    }
}
