<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Dto;

use apexl\dto\Dto;

final readonly class TabulatorAjaxColumnDefinitionDto extends Dto
{
    public function __construct(
        public string $primaryKey = 'id',
        public array $additionalFields = []
    ) {}

    public static function create(
        string $primaryKey = 'id',
        array $additionalFields = []

    ): self {
        return new self(...func_get_args());
    }
}