<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Dto;

use apexl\dto\Dto;

final readonly class TabulatorFieldTooltipDto extends Dto
{
    public function __construct(
        public string $field
    ) {}
}
