<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Enums;

use JsonSerializable;

enum TabulatorOnCellClickActionName implements JsonSerializable
{
    case FILTER_MAIN_TABLE;

    public function jsonSerialize(): string
    {
        return $this->name;
    }
}
