<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Interfaces;

use apexl\Io\modules\tabulator\Dto\TabulatorFilter;
use apexl\Vault\interfaces\driver;

interface TabulatableFilterQueryCallback
{
    public function __invoke(driver $query, TabulatorFilter $filter): driver;
}
