<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Interfaces;

interface TabulatableColumnCallback
{
    public function __invoke(HasTabulatorViewInterface $entity, string $field): mixed;
}
