<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Interfaces;

use JsonSerializable;

interface TabulatorFilterTypeInterface extends JsonSerializable
{
    public function asString(): string;
}