<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Collections;

use apexl\Io\modules\tabulator\Dto\TabulatorActionDto;
use apexl\Io\modules\tabulator\Interfaces\TabulatorAction;
use Arrayy\Collection\AbstractCollection;

class TabulatorActionCollection extends AbstractCollection
{

    public function getType(): string
    {
        return TabulatorAction::class;
    }
}