<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Dto;

use apexl\dto\Dto;
use apexl\Io\modules\tabulator\Enums\TabulatorOnRowClickActionName;

final readonly class TabulatorOnRowClickAction extends Dto
{
    public function __construct(
        public TabulatorOnRowClickActionName $action,
        public array $data,
    ) {}
}
