<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Collections;

use apexl\Io\modules\tabulator\Dto\TabulatorColumnDto;
use Arrayy\Collection\AbstractCollection;

/**
 * @extends  AbstractCollection<array-key,TabulatorColumnDto>
 */
class TabulatorColumnDtoCollection extends AbstractCollection
{
    public function getType(): string
    {
        return TabulatorColumnDto::class;
    }
}
