<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Dto;

class TabulatorColumnGroup
{
    public function __construct(

        public int $id,
        public string $title,

    ) {}
}

