<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Dto\RowFormatterDto;

use apexl\dto\Dto;

readonly abstract class TabulatorRowFormatterDto extends Dto
{
    public function __construct(
        public array $rowStyles,
        public array $rowClasses,
        public array $cellStyles,
        public array $cellClasses,
    ) {}
}