<?php

declare (strict_types=1);

namespace apexl\Io\modules\tabulator\Enums;

use JsonSerializable;

enum TabulatorMutator implements JsonSerializable
{
    case TRIM;

    case MONEY;

    public function jsonSerialize(): string
    {
        return strtolower($this->name);
    }
}
