<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Interfaces;

use apexl\Io\dto\LoaderSort;
use apexl\Vault\enums\SortDirection;

interface TabulatableSortQueryCallback
{
    public function __invoke(string $field, SortDirection $direction): LoaderSort;
}
