<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Services;

final readonly class TabulatorFieldRenamer
{
    public function rename(string $field): string
    {
        return str_replace('.', '__', $field);
    }
}