<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Dto;

use apexl\dto\Dto;
use Arrayy\Type\ArrayCollection;

final readonly class TabulatorData extends Dto
{
    public function __construct(
        public ArrayCollection $data,
        public int $last_page,
        public int $last_row
    ) {}
}
