<?php

namespace apexl\Io\modules\tabulator\Interfaces;

interface HasSubEntitiesInterface
{
    public function getSubEntityFieldsByKey(string $key): array;

    public function subEntityFields(): array;

    public function subEntityDerivedColumn(): string;
}
