<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\Interfaces;

use apexl\Io\interfaces\LoaderConditionInterface;
use apexl\Io\modules\tabulator\Dto\TabulatorFilter;

interface TabulatableLoaderConditionCallback
{
    public function __invoke(
        TabulatorFilter $filter,
    ): LoaderConditionInterface;
}
