<?php

declare(strict_types=1);

namespace apexl\Io\modules\tabulator\TabulatorActions;

final readonly class Link extends AbstractTabulatorAction
{
    public function __construct(
        protected string $link,
        protected string $icon,
        protected bool $newWindow = false,
        protected string $title = '',
        protected string $columnTitle = '',
        protected ?string $condition = null,
        protected bool $frozen = true,
    ) {}
}
